"
jv:vdb - Visual / VM Debugger
Copyright (C) 2015-now Jan Vrany

This software is licensed under 'Creative Commons Attribution-NonCommercial 4.0 International License'

You may find a full license text in LICENSE.txt or at http://creativecommons.org/licenses/by-nc/4.0/
"
"{ Package: 'jv:vdb' }"

"{ NameSpace: Smalltalk }"

TerminalView subclass:#VDBSimpleDebuggerConsoleView
	instanceVariableNames:'lastTabPressTime completionBlock completions'
	classVariableNames:''
	poolDictionaries:''
	category:'VDB-UI-Console'
!

!VDBSimpleDebuggerConsoleView class methodsFor:'documentation'!

copyright
"
jv:vdb - Visual / VM Debugger
Copyright (C) 2015-now Jan Vrany

This software is licensed under 'Creative Commons Attribution-NonCommercial 4.0 International License'

You may find a full license text in LICENSE.txt or at http://creativecommons.org/licenses/by-nc/4.0/
"
! !

!VDBSimpleDebuggerConsoleView methodsFor:'accessing'!

completionBlock
    ^ completionBlock
!

completionBlock:aBlock
    completionBlock := aBlock.

    "Modified: / 16-01-2019 / 11:35:56 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified: / 16-01-2019 / 14:43:18 / jv"
! !

!VDBSimpleDebuggerConsoleView methodsFor:'completion'!

completeCommand
    completionBlock notNil ifTrue:[
        completions := nil.
        completionBlock value: (lineBuffer copyTo: lineBufferCursorPosition - 1)
    ].

    "Created: / 16-01-2019 / 11:34:10 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified: / 16-01-2019 / 11:45:21 / jv"
!

completeCommandWith: anArray"of Strings"
    | completion |

    completions := anArray.
    completions isEmptyOrNil ifTrue:[
        completions := nil.
        self flash.
        ^self.
    ].
    completion := completions longestCommonPrefix copyFrom: lineBufferCursorPosition.

    completion isEmpty ifTrue:[ 
        self flash
    ] ifFalse:[
        self doInsertString: completion.
    ]

    "Created: / 16-01-2019 / 11:35:14 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified: / 16-01-2019 / 14:28:41 / jv"
!

completionShow
    "Shows completion candidates"

    | currentLineNr currentLine |

    completions isEmptyOrNil ifTrue:[ ^ self ].
    currentLineNr := cursorLine.
    currentLine := self list at: currentLineNr.
    self nextPutCRLF.
    completions do:[:each |
        self nextPutAll: each.
        self nextPutCRLF.
    ].
    self nextPutAll: currentLine.

    "Created: / 16-01-2019 / 11:58:44 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified: / 16-01-2019 / 12:39:48 / jv"
! !

!VDBSimpleDebuggerConsoleView methodsFor:'event handling'!

keyPress:aKey x:x y:y
    <resource: #keyboard (#Tab)>

    inStream isNil ifTrue:[^ self].

    aKey == #Tab ifTrue:[ 
        DebugKeyboard ifTrue:[
            Transcript showCR:'----'; show:'keyPress:' ; showCR:aKey printString.
        ].
        (lastTabPressTime notNil and: [(OperatingSystem getMillisecondTime - lastTabPressTime) < 1000]) ifTrue:[ 
            lastTabPressTime := nil.
            ^ self doTabTab.
        ] ifFalse:[ 
            lastTabPressTime := OperatingSystem getMillisecondTime .    
            ^ self doTab
        ].
    ].
    lastTabPressTime := nil.
    ^ super keyPress:aKey x:x y:y




    "
     DebugKeyboard := true
    "

    "Created: / 15-09-2016 / 23:55:56 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified: / 04-04-2018 / 22:57:53 / jv"
    "Modified: / 16-01-2019 / 12:31:55 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!VDBSimpleDebuggerConsoleView methodsFor:'functions'!

doInsertString: stringToComplete
    "Insert (add) `stringToComplete` at current linebuffer insert point.
     This is meant to implement command completion."

    lineBuffer := (lineBuffer copyTo:lineBufferCursorPosition-1)
                      , stringToComplete
                      , (lineBuffer copyFrom:lineBufferCursorPosition).
    self insertStringAtCursor:stringToComplete.
    lineBufferCursorPosition := lineBufferCursorPosition + stringToComplete size.

    "Created: / 31-12-2018 / 09:26:08 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

doTab
    self completeCommand

    "Created: / 15-09-2016 / 23:56:12 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified: / 04-04-2018 / 22:58:55 / jv"
    "Modified: / 16-01-2019 / 11:31:57 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

doTabTab
    completionBlock notNil ifTrue:[ 
        self completionShow
    ].

    "Created: / 16-09-2016 / 00:56:02 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified: / 04-04-2018 / 22:59:02 / jv"
    "Modified: / 16-01-2019 / 12:00:23 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!VDBSimpleDebuggerConsoleView methodsFor:'initialization'!

initialize
    "Invoked when a new instance is created."

    "/ please change as required (and remove this comment)
    super initialize.
    lastTabPressTime := nil.
    completions := nil.

    "Modified: / 16-01-2019 / 11:59:02 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!VDBSimpleDebuggerConsoleView methodsFor:'processing-input'!

nextPut:char
    super nextPut:char.
    "/ Following is a hack to make dumb (simple) console to remember
    "/ the input. Not sure why this is needed but well, it appears
    "/ to work. This code is a bit of a mess.
    (char == Character nl) ifTrue:[
        rangeEndLine := rangeEndLine - 1
    ]

    "Created: / 14-12-2018 / 10:07:12 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

nextPutCRLF
    self pushEvent:#processInput:n: with: Character return , Character linefeed with:2

    "Created: / 16-01-2019 / 12:45:16 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!VDBSimpleDebuggerConsoleView class methodsFor:'documentation'!

version_HG

    ^ '$Changeset: <not expanded> $'
! !

