"
jv:vdb - Visual / VM Debugger
Copyright (C) 2015-now Jan Vrany

This software is licensed under 'Creative Commons Attribution-NonCommercial 4.0 International License'

You may find a full license text in LICENSE.txt or at http://creativecommons.org/licenses/by-nc/4.0/
"
"{ Package: 'jv:vdb/tests' }"

"{ NameSpace: Smalltalk }"

TestCase subclass:#VDBSimpleDebuggerConsoleViewTest
	instanceVariableNames:'preferences consoleOutput consoleInput console consoleI'
	classVariableNames:''
	poolDictionaries:''
	category:'VDB-UI-Console-Tests'
!

!VDBSimpleDebuggerConsoleViewTest class methodsFor:'documentation'!

copyright
"
jv:vdb - Visual / VM Debugger
Copyright (C) 2015-now Jan Vrany

This software is licensed under 'Creative Commons Attribution-NonCommercial 4.0 International License'

You may find a full license text in LICENSE.txt or at http://creativecommons.org/licenses/by-nc/4.0/
"
! !

!VDBSimpleDebuggerConsoleViewTest methodsFor:'running'!

setUp
    | window |

    super setUp.

    Screen current isNil ifTrue:[ Smalltalk openDisplay ].
    self skipIf: Screen current isNil description: 'No display connection'.
    Smalltalk loadPackage: 'stx:goodies/sunit/ext/ui'.

    consoleOutput := GDBInternalPipeStream new.
    consoleInput := GDBInternalPipeStream new.

    window := StandardSystemView new.
    window extent: 320 @ 200.
    window label: self printString.
    console := VDBSimpleDebuggerConsoleView origin: 0.0@0.0 extent: 1.0@1.0 in: window.

    "/ Following should be kept in sync with
    "/ VDBSimpleDebuggerConsoleApplication >> consoleView
    console localEcho:true.
    console inputTranslateCRToNL:true.
    console lineEditMode:true.
    console inStream:consoleInput.
    console outStream:consoleOutput.
    console startReaderProcessWhenVisible.    

    window open.
    window waitUntilVisible.

    consoleI := console interactor

    "Modified: / 16-01-2019 / 10:24:12 / jv"
    "Modified: / 16-01-2019 / 16:23:28 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

tearDown
    super tearDown.
    console topView destroy.

    "Created: / 28-09-2018 / 08:29:43 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified: / 16-01-2019 / 16:23:35 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!VDBSimpleDebuggerConsoleViewTest methodsFor:'tests'!

test_completion_01

    console completionBlock:[ :partial | 
        console completeCommandWith: #('finish')
    ].
    consoleI type: 'fi'.
    consoleI type: #Tab.
    self assert: console list first asString = 'finish'.

    "Created: / 16-01-2019 / 11:44:32 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

test_completion_02

    console completionBlock:[ :partial | 
        console completeCommandWith:     
            (#('file' 'finish') select: [ :each | each startsWith: partial ]).
    ].
    consoleI type: 'f'.
    consoleI type: #Tab.
    self assert: console list first asString = 'fi'.

    consoleI type: 'l'.
    consoleI type: #Tab.
    self assert: console list first asString = 'file'.

    "Created: / 16-01-2019 / 11:46:43 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

test_completion_03
    consoleOutput nextPutAll: '-> '.
    [ consoleOutput hasData ] whileTrue:[ Delay waitForMilliseconds: 100 ].

    console completionBlock:[ :partial | 
        console completeCommandWith:     
            (#('file' 'finish') select: [ :each | each startsWith: partial ]).
    ].
    consoleI type: 'f'.
    consoleI type: #Tab.
    consoleI type: #Tab.
    self assert: (console list at: 1) asString = '-> fi'.
    self assert: (console list at: 2) asString = 'file'.
    self assert: (console list at: 3) asString = 'finish'.
    self assert: (console list at: 4) asString = '-> fi'.

    consoleI type: 'n'.
    consoleI type: #Tab.
    self assert: (console list at: 4) asString = '-> finish'.

    "Created: / 16-01-2019 / 11:56:11 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified: / 16-01-2019 / 12:51:36 / jv"
!

test_io_01

    consoleOutput nextPutAll: 'Hello World'; crlf; nextPutAll: '> '.
    [ consoleOutput hasData ] whileTrue:[ Delay waitForMilliseconds: 100 ].

    self assert: console list first = 'Hello World'.
    self assert: console list second = '> '.

    consoleI type: 'command'.
    self assert: console list second = '> command'.

    consoleI type: #Return.
    self assert: consoleInput nextLine asString = 'command'.

    "Created: / 16-01-2019 / 10:23:56 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified: / 16-01-2019 / 10:36:49 / jv"
! !

!VDBSimpleDebuggerConsoleViewTest class methodsFor:'documentation'!

version_HG

    ^ '$Changeset: <not expanded> $'
! !

